#!/usr/bin/env bash

#
# Auto GIT gnome extension
# https://jasonmun.blogspot.my
# https://github.com/yomun/auto-git
# 
# Copyright (C) 2017 Jason Mun
#
# Auto GIT gnome extension is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Auto GIT gnome extension is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Auto GIT gnome extension.  If not, see <http://www.gnu.org/licenses/>.
# 
#

SHELL_PATH="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

DATA_PATH=`echo "${SHELL_PATH}" | sed -e 's/\/sh$//'`

SYMBOL=`echo ${LANGUAGE} | grep -c "zh_CN"`
	
if [ ${SYMBOL} -eq 1 ]; then SYMBOL="："; else SYMBOL=":"; fi

function CAN_COMMIT_FILES()
{
	GIT_STATUS=`git -C "${iPath}" status`; echo "${GIT_STATUS}" > "${DATA_PATH}/status.txt"
	
	EMPTY_LINE_NUM_LIST=`cat "${DATA_PATH}/status.txt" | grep -n '^$' | sed -e "s/://g" | sed -e "s/：//g" | tr "\n" " "`
	
	if [ ${#EMPTY_LINE_NUM_LIST} -gt 0 ]
	then		
		local CNT=$((0))

		for num in ${EMPTY_LINE_NUM_LIST}
		do
			CNT=$((CNT + 1))

			if [ $((CNT % 2)) -eq 0 ]; then continue; fi

			N=$((num))
			
			PROCESS_RENAME
			
			CNT_COMMIT=$((0))
			
			PROCESS_CONTENTS
			
			if [ ${CNT_COMMIT} -gt 0 ]
			then	
				local DT=`date '+%Y-%m-%d %H:%M'`

				git -C "${iPath}" commit -m "${DT}"

				local branch_name=`git -C "${iPath}" branch --no-color | cut -c3-`

				git -C "${iPath}" push origin ${branch_name}

				local REPOSITORY=`echo "${SSH_URL}" | sed "s/^.*://g"`

				REPO_ID=`echo "${REPOSITORY}" | sed "s/^.*\///g"`

				sed -i "/${REPO_ID}/d" "${DATA_PATH}/update.log"

				echo "${DT} ${REPOSITORY}" >> "${DATA_PATH}/update.log"
			fi
		done
	fi
}

function PROCESS_RENAME()
{
	if [ `cat "${DATA_PATH}/status.txt" | sed -n "$((N - 1))p" | grep -c "git reset HEAD "` -gt 0 ]
	then
		local cnt=$((0))
	
		local content="start"

		while [ ${#content} -gt 0 ]
		do
			N=$((N + 1))
			
			content=`cat "${DATA_PATH}/status.txt" | sed -n "${N}p" | xargs`
			
			# Rename: test/test.txt -> test2/test.txt
			if [ `echo "${content}" | grep -c " -> "` -gt 0 ]
			then
				git -C "${iPath}" reset HEAD "${content}"
				echo ">> git -C '${iPath}' reset HEAD '${content}'"
				
				git -C "${iPath}" rm "${content}"
				echo ">> git -C '${iPath}' rm '${content}'"
				
				cnt=$((cnt + 1))
			fi
		done
	fi
}

function PROCESS_CONTENTS()
{
	local content="start"

	while [ ${#content} -gt 0 ]
	do		
		N=$((N+1))

		content=`cat "${DATA_PATH}/status.txt" | sed -n "${N}p" | xargs`

		if [ ${#content} -gt 0 ]
		then
			if [ `echo "${content}" | grep -c "${SYMBOL}"` -gt 0 ]
			then				
				# update / delete file
				content=`echo "${content}" | sed -e "s/.*${SYMBOL} //g"`

				if [ -f "${iPath}/${content}" ]
				then
					# update file
					git -C "${iPath}" add "${content}"
					echo ">> git -C '${iPath}' add '${content}'"
					CNT_COMMIT=$((CNT_COMMIT + 1))
				else
					# delete file
					git -C "${iPath}" rm "${content}"
					echo ">> git -C '${iPath}' rm '${content}'"
					CNT_COMMIT=$((CNT_COMMIT + 1))
				fi
			else
				# new file / folder
				if [ "${content: -1}" = "/" ]
				then
					# new folder
					if [ -d "${iPath}/${content}" ]
					then
						git -C "${iPath}" add "${content}"
						echo ">> git -C '${iPath}' add '${content}'"
						CNT_COMMIT=$((CNT_COMMIT + 1))
					fi
				else
					# new file
					if [ -f "${iPath}/${content}" ]
					then
						git -C "${iPath}" add "${content}"
						echo ">> git -C '${iPath}' add '${content}'"
						CNT_COMMIT=$((CNT_COMMIT + 1))
					fi
				fi
			fi
		fi
	done
}


PATH_LIST=`find "/home/${USER}" -name ".git" 2>&1 | grep -v '^find: '`

for git_iPath in ${PATH_LIST}
do	
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/.git"`       -gt 0 ]; then continue; fi
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/Downloads/"` -gt 0 ]; then continue; fi
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/Music/"`     -gt 0 ]; then continue; fi
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/Pictures/"`  -gt 0 ]; then continue; fi
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/Public/"`    -gt 0 ]; then continue; fi
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/Templates/"` -gt 0 ]; then continue; fi
	if [ `echo "${git_iPath}" | grep -c "/home/${USER}/Videos/"`    -gt 0 ]; then continue; fi
	
	iPath=`echo "${git_iPath}" | sed 's/\/.git//g'`
	
	echo "[${iPath}]"
	
	SSH_URL=`git -C "${iPath}" remote get-url origin`
	
	if [ `echo "${SSH_URL}" | grep -c "git@github.com:"` -gt 0 ]
	then
		CAN_COMMIT_FILES
	fi
done
